"""sec-value

Revision ID: f7b416c59e27
Revises: 0a4bbaa82cd1
Create Date: 2025-11-01 15:30:24.903828

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "f7b416c59e27"
down_revision: Union[str, None] = "0a4bbaa82cd1"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("sec", schema=None) as batch_op:
        batch_op.add_column(sa.Column("value", sa.Float(), nullable=True))

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("sec", schema=None) as batch_op:
        batch_op.drop_column("value")

    # ### end Alembic commands ###
